/// <reference types="node" />
import { NuxtAppOptions, NuxtError, Context, Module } from '@nuxt/types';
import { Ref, defineComponent as defineComponent$1 } from '@vue/composition-api';
export { ComponentRenderProxy, InjectionKey, PropOptions, PropType, Ref, SetupContext, VueWatcher, computed, createComponent, createElement, getCurrentInstance, inject, isRef, onActivated, onBeforeMount, onBeforeUnmount, onBeforeUpdate, onDeactivated, onErrorCaptured, onMounted, onServerPrefetch, onUnmounted, onUpdated, provide, reactive, ref, set, toRefs, watch, watchEffect } from '@vue/composition-api';
import { Dictionary, Component } from 'vue-router/types/router';
import { Location, Route } from 'vue-router';
import { ServerResponse } from 'http';
import { IncomingMessage } from 'connect';
import { Store } from 'vuex/types/index';
import { ComponentInstance } from '@vue/composition-api/dist/component';
import { MetaInfo } from 'vue-meta';

/**
 * You can create reactive values that depend on asynchronous calls with `useAsync`.
 * On the server, this helper will inline the result of the async call in your HTML and automatically inject them into your client code. Much like `asyncData`, it _won't_ re-run these async calls client-side.
 *
 * However, if the call hasn't been carried out on SSR (such as if you have navigated to the page after initial load), it returns a `null` ref that is filled with the result of the async call when it resolves.
 *
 * **At the moment, `useAsync` is only suitable for one-offs, unless you provide your own unique key.**
 * @param cb The async function that will populate the ref this function returns.
 * @param key Under the hood, `useAsync` requires a key to ensure that the ref values match between client and server. If you have added `nuxt-composition-api` to your `buildModules`, this will be done automagically by an injected Babel plugin. If you need to do things differently, you can specify a key manually or add `nuxt-composition-api/babel` to your Babel plugins.
 *
 * @example
  ```ts
  import { defineComponent, useAsync, computed } from 'nuxt-composition-api'
  import axios from 'axios'

  export default defineComponent({
    setup() {
      const posts = useAsync(() => axios.get('/api/posts'))

      return { posts }
    },
  })
  ```
 */
declare const useAsync: <T>(cb: () => T | Promise<T>, key?: string | Ref<null> | undefined) => Ref<T | null>;

/**
 * If you want to enable `useMeta`, make sure to include `head: {}` in your component definition.
 * @example
  ```ts
  import { defineComponent } from 'nuxt-composition-api'

  export default defineComponent({
    head: {},
    setup() {
      ...
    }
  })
  ```
 */
declare const defineComponent: typeof defineComponent$1;

interface ContextCallback {
    (context: Context): void;
}
/**
 * @deprecated
 * Recommend using `useContext` instead
 */
declare const withContext: (callback: ContextCallback) => void;
/**
 * `useContext` which will return the Nuxt context.
 * @example
  ```ts
  import { defineComponent, ref, useContext } from 'nuxt-composition-api'

  export default defineComponent({
    setup() {
      const { store } = useContext()
      store.dispatch('myAction')
    },
  })
  ```
 */
declare const useContext: () => {
    route: Readonly<Ref<Readonly<import("vue-router").Route>>>;
    query: Readonly<Ref<Readonly<import("vue-router/types/router").Dictionary<string | (string | null)[]>>>>;
    from: Readonly<Ref<string | undefined>>;
    params: Readonly<Ref<Readonly<import("vue-router/types/router").Dictionary<string>>>>;
    app: NuxtAppOptions;
    base: string;
    isClient: boolean;
    isServer: boolean;
    isStatic: boolean;
    isDev: boolean;
    isHMR: boolean;
    store: Store<any>;
    env: Record<string, any>;
    payload: any;
    req: IncomingMessage;
    res: ServerResponse;
    redirect(status: number, path: string, query?: Dictionary<string | (string | null)[]> | undefined): void;
    redirect(path: string, query?: Dictionary<string | (string | null)[]> | undefined): void;
    redirect(location: Location): void;
    error(params: NuxtError): void;
    nuxtState: Record<string, any>;
    beforeNuxtRender(fn: (params: {
        Components: (to?: string | Route | Location | undefined) => Component[];
        nuxtState: Record<string, any>;
    }) => void): void;
};

interface Fetch {
    (context: ComponentInstance): void;
}
/**
 * Versions of Nuxt newer than v2.12 support a [custom hook called `fetch`](https://nuxtjs.org/api/pages-fetch/) that allows server-side and client-side asynchronous data-fetching.

 * @param callback The async function you want to run.
 * @example

  ```ts
  import { defineComponent, ref, useFetch } from 'nuxt-composition-api'
  import axios from 'axios'

  export default defineComponent({
    setup() {
      const name = ref('')

      const { $fetch, $fetchState } = useFetch(async () => {
        name.value = await axios.get('https://myapi.com/name')
      })

      // Manually trigger a refetch
      $fetch()

      // Access fetch error, pending and timestamp
      $fetchState

      return { name }
    },
  })
  ```
 */
declare const useFetch: (callback: Fetch) => {
    $fetch: () => void;
    $fetchState: {
        error: Error | null;
        pending: boolean;
        timestamp: number;
    };
};

declare type MetaInfoMapper<T> = {
    [P in keyof T]: P extends 'base' ? T[P] | undefined : T[P] extends Function ? T[P] | undefined : T[P] extends Array<any> | Record<string, any> ? T[P] : T[P] | undefined;
};
declare type ToRefs<T extends Record<string, any>> = {
    [P in keyof T]: Ref<T[P]>;
};
/**
 * `useMeta` lets you interact directly with [`head()` properties](https://nuxtjs.org/api/pages-head/) in `setup`. **Make sure you set `head: {}` in your component options.**
 * @example
    ```ts
    import { defineComponent, useMeta, computed } from 'nuxt-composition-api'

    export default defineComponent({
      head: {},
      setup() {
        const { title } = useMeta()
        title.value = 'My page'
      })
    })
    ```
 * @param init Whatever defaults you want to set for `head` properties.
 */
declare const useMeta: <T extends MetaInfo>(init?: T | undefined) => ToRefs<MetaInfoMapper<Required<MetaInfo>> & T>;

declare function setSSRContext(ssrContext: any): void;
/**
 * `ssrRef` will automatically add ref values to `window.__NUXT__` on SSR if they have been changed from their initial value. It can be used outside of components, such as in shared utility functions, and it supports passing a factory function that will generate the initial value of the ref. **At the moment, an `ssrRef` is only suitable for one-offs, unless you provide your own unique key.**
 * @param value This can be an initial value or a factory function that will be executed on server-side to get the initial value.
 * @param key Under the hood, `ssrRef` requires a key to ensure that the ref values match between client and server. If you have added `nuxt-composition-api` to your `buildModules`, this will be done automagically by an injected Babel plugin. If you need to do things differently, you can specify a key manually or add `nuxt-composition-api/babel` to your Babel plugins.
 * @example
  ```ts
  import { ssrRef } from 'nuxt-composition-api'

  const val = ssrRef('')

  // When hard-reloaded, `val` will be initialised to 'server set'
  if (process.server) val.value = 'server set'

  // When hard-reloaded, the result of myExpensiveSetterFunction() will
  // be encoded in nuxtState and used as the initial value of this ref.
  // If client-loaded, the setter function will run to come up with initial value.
  const val2 = ssrRef(myExpensiveSetterFunction)
  ```
 */
declare const ssrRef: <T>(value: T | (() => T), key?: string | undefined) => Ref<T>;
/**
 * This helper creates a [`shallowRef`](https://vue-composition-api-rfc.netlify.app/api.html#shallowref) (a ref that tracks its own .value mutation but doesn't make its value reactive) that is synced between client & server.
 * @param value This can be an initial value or a factory function that will be executed on server-side to get the initial value.
 * @param key Under the hood, `shallowSsrRef` requires a key to ensure that the ref values match between client and server. If you have added `nuxt-composition-api` to your `buildModules`, this will be done automagically by an injected Babel plugin. If you need to do things differently, you can specify a key manually or add `nuxt-composition-api/babel` to your Babel plugins.
 
 * @example
  ```ts
  import { shallowSsrRef, onMounted } from 'nuxt-composition-api'

  const shallow = shallowSsrRef({ v: 'init' })
  if (process.server) shallow.value = { v: 'changed' }

  // On client-side, shallow.value will be { v: changed }
  onMounted(() => {
    // This and other changes outside of setup won't trigger component updates.
    shallow.value.v = 'Hello World'
  })
  ```
 */
declare const shallowSsrRef: <T>(value: T | (() => T), key?: string | undefined) => Ref<T>;

declare const compositionApiModule: Module<any>;
declare const meta: any;

export default compositionApiModule;
export { defineComponent, meta, setSSRContext, shallowSsrRef, ssrRef, useAsync, useContext, useFetch, useMeta, withContext };
