'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var path = require('path');
var compositionApi = require('@vue/composition-api');
var defu = _interopDefault(require('defu'));
var Vue = _interopDefault(require('vue'));

function getValue(value) {
    if (value instanceof Function)
        return value();
    return value;
}
let data = {};
function setSSRContext(ssrContext) {
    data = Object.assign({}, {});
    ssrContext.nuxt.ssrRefs = data;
}
const isProxyable = (val) => val && typeof val === 'object';
const sanitise = (val) => val;
/**
 * `ssrRef` will automatically add ref values to `window.__NUXT__` on SSR if they have been changed from their initial value. It can be used outside of components, such as in shared utility functions, and it supports passing a factory function that will generate the initial value of the ref. **At the moment, an `ssrRef` is only suitable for one-offs, unless you provide your own unique key.**
 * @param value This can be an initial value or a factory function that will be executed on server-side to get the initial value.
 * @param key Under the hood, `ssrRef` requires a key to ensure that the ref values match between client and server. If you have added `nuxt-composition-api` to your `buildModules`, this will be done automagically by an injected Babel plugin. If you need to do things differently, you can specify a key manually or add `nuxt-composition-api/babel` to your Babel plugins.
 * @example
  ```ts
  import { ssrRef } from 'nuxt-composition-api'

  const val = ssrRef('')

  // When hard-reloaded, `val` will be initialised to 'server set'
  if (process.server) val.value = 'server set'

  // When hard-reloaded, the result of myExpensiveSetterFunction() will
  // be encoded in nuxtState and used as the initial value of this ref.
  // If client-loaded, the setter function will run to come up with initial value.
  const val2 = ssrRef(myExpensiveSetterFunction)
  ```
 */
const ssrRef = (value, key) => {
    var _a, _b, _c;
    if (!key) {
        throw new Error("You must provide a key. You can have it generated automatically by adding 'nuxt-composition-api/babel' to your Babel plugins.");
    }
    if (process.client) {
        return compositionApi.ref((_c = (_b = (_a = window.__NUXT__) === null || _a === void 0 ? void 0 : _a.ssrRefs) === null || _b === void 0 ? void 0 : _b[key]) !== null && _c !== void 0 ? _c : getValue(value));
    }
    const val = getValue(value);
    const _ref = compositionApi.ref(val);
    if (value instanceof Function)
        data[key] = sanitise(val);
    const getProxy = (observable) => new Proxy(observable, {
        get(target, prop) {
            if (isProxyable(target[prop]))
                return getProxy(target[prop]);
            return Reflect.get(target, prop);
        },
        set(obj, prop, val) {
            data[key] = sanitise(_ref.value);
            return Reflect.set(obj, prop, val);
        },
    });
    const proxy = compositionApi.computed({
        get: () => (isProxyable(_ref.value) ? getProxy(_ref.value) : _ref.value),
        set: v => {
            data[key] = sanitise(v);
            _ref.value = v;
        },
    });
    return proxy;
};
// TODO: remove when https://github.com/vuejs/composition-api/pull/311 is merged
function shallowRef(value) {
    return compositionApi.computed({
        get: () => value,
        set: v => (value = v),
    });
}
/**
 * This helper creates a [`shallowRef`](https://vue-composition-api-rfc.netlify.app/api.html#shallowref) (a ref that tracks its own .value mutation but doesn't make its value reactive) that is synced between client & server.
 * @param value This can be an initial value or a factory function that will be executed on server-side to get the initial value.
 * @param key Under the hood, `shallowSsrRef` requires a key to ensure that the ref values match between client and server. If you have added `nuxt-composition-api` to your `buildModules`, this will be done automagically by an injected Babel plugin. If you need to do things differently, you can specify a key manually or add `nuxt-composition-api/babel` to your Babel plugins.
 
 * @example
  ```ts
  import { shallowSsrRef, onMounted } from 'nuxt-composition-api'

  const shallow = shallowSsrRef({ v: 'init' })
  if (process.server) shallow.value = { v: 'changed' }

  // On client-side, shallow.value will be { v: changed }
  onMounted(() => {
    // This and other changes outside of setup won't trigger component updates.
    shallow.value.v = 'Hello World'
  })
  ```
 */
const shallowSsrRef = (value, key) => {
    var _a, _b, _c;
    if (!key) {
        throw new Error("You must provide a key. You can have it generated automatically by adding 'nuxt-composition-api/babel' to your Babel plugins.");
    }
    if (process.client) {
        return shallowRef((_c = (_b = (_a = window.__NUXT__) === null || _a === void 0 ? void 0 : _a.ssrRefs) === null || _b === void 0 ? void 0 : _b[key]) !== null && _c !== void 0 ? _c : getValue(value));
    }
    let _val = getValue(value);
    if (value instanceof Function) {
        data[key] = sanitise(_val);
    }
    return compositionApi.computed({
        get: () => _val,
        set: v => {
            data[key] = sanitise(v);
            _val = v;
        },
    });
};

/**
 * You can create reactive values that depend on asynchronous calls with `useAsync`.
 * On the server, this helper will inline the result of the async call in your HTML and automatically inject them into your client code. Much like `asyncData`, it _won't_ re-run these async calls client-side.
 *
 * However, if the call hasn't been carried out on SSR (such as if you have navigated to the page after initial load), it returns a `null` ref that is filled with the result of the async call when it resolves.
 *
 * **At the moment, `useAsync` is only suitable for one-offs, unless you provide your own unique key.**
 * @param cb The async function that will populate the ref this function returns.
 * @param key Under the hood, `useAsync` requires a key to ensure that the ref values match between client and server. If you have added `nuxt-composition-api` to your `buildModules`, this will be done automagically by an injected Babel plugin. If you need to do things differently, you can specify a key manually or add `nuxt-composition-api/babel` to your Babel plugins.
 *
 * @example
  ```ts
  import { defineComponent, useAsync, computed } from 'nuxt-composition-api'
  import axios from 'axios'

  export default defineComponent({
    setup() {
      const posts = useAsync(() => axios.get('/api/posts'))

      return { posts }
    },
  })
  ```
 */
const useAsync = (cb, key) => {
    if (!key) {
        throw new Error("You must provide a key. You can have it generated automatically by adding 'nuxt-composition-api/babel' to your Babel plugins.");
    }
    const _ref = compositionApi.isRef(key) ? key : ssrRef(null, key);
    if (!_ref.value) {
        const p = cb();
        if (p instanceof Promise) {
            if (process.server) {
                compositionApi.onServerPrefetch(async () => {
                    _ref.value = await p;
                });
            }
            else {
                // eslint-disable-next-line
                p.then(res => (_ref.value = res));
            }
        }
        else {
            _ref.value = p;
        }
    }
    return _ref;
};

function createEmptyMeta() {
    return {
        __dangerouslyDisableSanitizers: [],
        __dangerouslyDisableSanitizersByTagID: {},
        title: undefined,
        titleTemplate: undefined,
        htmlAttrs: {},
        headAttrs: {},
        bodyAttrs: {},
        base: undefined,
        meta: [],
        link: [],
        style: [],
        script: [],
        noscript: [],
        changed: undefined,
        afterNavigation: undefined,
    };
}
const getHeadOptions = (options) => {
    const _head = compositionApi.reactive({});
    if (!(options.head instanceof Function)) {
        Object.assign(_head, options.head);
    }
    const head = options.head instanceof Function
        ? () => defu(_head, options.head())
        : () => _head;
    return { _head, head };
};
/**
 * `useMeta` lets you interact directly with [`head()` properties](https://nuxtjs.org/api/pages-head/) in `setup`. **Make sure you set `head: {}` in your component options.**
 * @example
    ```ts
    import { defineComponent, useMeta, computed } from 'nuxt-composition-api'

    export default defineComponent({
      head: {},
      setup() {
        const { title } = useMeta()
        title.value = 'My page'
      })
    })
    ```
 * @param init Whatever defaults you want to set for `head` properties.
 */
const useMeta = (init) => {
    const vm = compositionApi.getCurrentInstance();
    if (!vm)
        throw new Error('useMeta must be called within a component.');
    if (!('_head' in vm.$options))
        throw new Error('In order to enable `useMeta`, please make sure you include `head: {}` within your component definition, and you are using the `defineComponent` exported from nuxt-composition-api.');
    const { _head } = vm.$options;
    Object.assign(_head, createEmptyMeta());
    Object.assign(_head, init || {});
    return compositionApi.toRefs(_head);
};

/**
 * If you want to enable `useMeta`, make sure to include `head: {}` in your component definition.
 * @example
  ```ts
  import { defineComponent } from 'nuxt-composition-api'

  export default defineComponent({
    head: {},
    setup() {
      ...
    }
  })
  ```
 */
const defineComponent = (options) => {
    if (!('head' in options))
        return options;
    return {
        ...options,
        ...getHeadOptions(options),
    };
};

/**
 * @deprecated
 * Recommend using `useContext` instead
 */
const withContext = (callback) => {
    const vm = compositionApi.getCurrentInstance();
    if (!vm)
        throw new Error('This must be called within a setup function.');
    callback(vm.$nuxt.context);
};
/**
 * `useContext` which will return the Nuxt context.
 * @example
  ```ts
  import { defineComponent, ref, useContext } from 'nuxt-composition-api'

  export default defineComponent({
    setup() {
      const { store } = useContext()
      store.dispatch('myAction')
    },
  })
  ```
 */
const useContext = () => {
    const vm = compositionApi.getCurrentInstance();
    if (!vm)
        throw new Error('This must be called within a setup function.');
    return {
        ...vm.$nuxt.context,
        route: compositionApi.computed(() => vm.$route),
        query: compositionApi.computed(() => vm.$route.query),
        from: compositionApi.computed(() => vm.$route.redirectedFrom),
        params: compositionApi.computed(() => vm.$route.params),
    };
};

function normalizeError(err) {
    let message;
    if (!(err.message || typeof err === 'string')) {
        try {
            message = JSON.stringify(err, null, 2);
        }
        catch (e) {
            message = `[${err.constructor.name}]`;
        }
    }
    else {
        message = err.message || err;
    }
    return {
        ...err,
        message,
        statusCode: err.statusCode ||
            err.status ||
            (err.response && err.response.status) ||
            500,
    };
}
const fetches = new WeakMap();
const isSsrHydration = (vm) => { var _a, _b, _c; return (_c = (_b = (_a = vm.$vnode) === null || _a === void 0 ? void 0 : _a.elm) === null || _b === void 0 ? void 0 : _b.dataset) === null || _c === void 0 ? void 0 : _c.fetchKey; };
const nuxtState = process.client && window.__NUXT__;
function registerCallback(vm, callback) {
    const callbacks = fetches.get(vm) || [];
    fetches.set(vm, [...callbacks, callback]);
}
async function callFetches() {
    const fetchesToCall = fetches.get(this);
    if (!fetchesToCall)
        return;
    this.$nuxt.nbFetching++;
    this.$fetchState.pending = true;
    this.$fetchState.error = null;
    this._hydrated = false;
    let error = null;
    const startTime = Date.now();
    try {
        await Promise.all(fetchesToCall.map(fetch => fetch(this)));
    }
    catch (err) {
        error = normalizeError(err);
    }
    const delayLeft = (this._fetchDelay || 0) - (Date.now() - startTime);
    if (delayLeft > 0) {
        await new Promise(resolve => setTimeout(resolve, delayLeft));
    }
    this.$fetchState.error = error;
    this.$fetchState.pending = false;
    this.$fetchState.timestamp = Date.now();
    this.$nextTick(() => this.$nuxt.nbFetching--);
}
async function serverPrefetch(vm) {
    if (!vm._fetchOnServer) {
        return;
    }
    // Call and await on $fetch
    vm.$fetchState =
        vm.$fetchState ||
            Vue.observable({
                error: null,
                pending: false,
                timestamp: 0,
            });
    try {
        await callFetches.call(vm);
    }
    catch (err) {
        vm.$fetchState.error = normalizeError(err);
    }
    vm.$fetchState.pending = false;
    // Define an ssrKey for hydration
    vm._fetchKey = vm.$ssrContext.nuxt.fetch.length;
    // Add data-fetch-key on parent element of Component
    if (!vm.$vnode.data)
        vm.$vnode.data = {};
    const attrs = (vm.$vnode.data.attrs = vm.$vnode.data.attrs || {});
    attrs['data-fetch-key'] = vm._fetchKey;
    // Add to ssrContext for window.__NUXT__.fetch
    vm.$ssrContext.nuxt.fetch.push(vm.$fetchState.error
        ? { _error: vm.$fetchState.error }
        : JSON.parse(JSON.stringify(vm._data)));
}
/**
 * Versions of Nuxt newer than v2.12 support a [custom hook called `fetch`](https://nuxtjs.org/api/pages-fetch/) that allows server-side and client-side asynchronous data-fetching.

 * @param callback The async function you want to run.
 * @example

  ```ts
  import { defineComponent, ref, useFetch } from 'nuxt-composition-api'
  import axios from 'axios'

  export default defineComponent({
    setup() {
      const name = ref('')

      const { $fetch, $fetchState } = useFetch(async () => {
        name.value = await axios.get('https://myapi.com/name')
      })

      // Manually trigger a refetch
      $fetch()

      // Access fetch error, pending and timestamp
      $fetchState

      return { name }
    },
  })
  ```
 */
const useFetch = (callback) => {
    var _a;
    const vm = compositionApi.getCurrentInstance();
    if (!vm)
        throw new Error('This must be called within a setup function.');
    vm.$fetchState =
        vm.$fetchState ||
            Vue.observable({
                error: null,
                pending: false,
                timestamp: 0,
            });
    vm.$fetch = callFetches.bind(vm);
    registerCallback(vm, callback);
    if (typeof vm.$options.fetchOnServer === 'function') {
        vm._fetchOnServer = vm.$options.fetchOnServer.call(vm) !== false;
    }
    else {
        vm._fetchOnServer = vm.$options.fetchOnServer !== false;
    }
    compositionApi.onServerPrefetch(() => serverPrefetch(vm));
    compositionApi.onBeforeMount(() => {
        if (!vm._hydrated) {
            return callFetches.call(vm);
        }
    });
    if (process.server || !isSsrHydration(vm))
        return {
            $fetch: vm.$fetch,
            $fetchState: vm.$fetchState,
        };
    // Hydrate component
    vm._hydrated = true;
    vm._fetchKey = +((_a = vm.$vnode.elm) === null || _a === void 0 ? void 0 : _a.dataset.fetchKey);
    const data = nuxtState.fetch[vm._fetchKey];
    // If fetch error
    if (data && data._error) {
        vm.$fetchState.error = data._error;
        return {
            $fetch: vm.$fetch,
            $fetchState: vm.$fetchState,
        };
    }
    compositionApi.onBeforeMount(() => {
        // Merge data
        for (const key in data) {
            try {
                Vue.set(vm, key, data[key]);
            }
            catch (e) {
                if (process.env.NODE_ENV === 'development')
                    // eslint-disable-next-line
                    console.warn(`Could not hydrate ${key}.`);
            }
        }
    });
    return {
        $fetch: vm.$fetch,
        $fetchState: vm.$fetchState,
    };
};

const compositionApiModule = function () {
    const libRoot = path.resolve(__dirname, '..');
    let corejsPolyfill;
    try {
        if (!this.options.modern) {
            // eslint-disable-next-line
            const corejsPkg = require('core-js/package.json');
            corejsPolyfill = corejsPkg.version.slice(0, 1);
        }
    }
    catch (_a) {
        corejsPolyfill = undefined;
    }
    const { dst } = this.addTemplate({
        src: path.resolve(libRoot, 'lib', 'plugin.js'),
        fileName: path.join('composition-api', 'plugin.js'),
        options: {
            corejsPolyfill,
        },
    });
    this.options.build = this.options.build || {};
    this.options.build.babel = this.options.build.babel || {};
    this.options.build.babel.plugins = this.options.build.babel.plugins || [];
    this.options.build.babel.plugins.push(path.join(__dirname, 'babel'));
    this.options.build.transpile = this.options.build.transpile || [];
    this.options.build.transpile.push(/nuxt-composition-api/);
    this.options.plugins = this.options.plugins || [];
    this.options.plugins.unshift(path.resolve(this.options.buildDir || '', dst));
};
// eslint-disable-next-line
const meta = require('../package.json');

Object.defineProperty(exports, 'computed', {
  enumerable: true,
  get: function () {
    return compositionApi.computed;
  }
});
Object.defineProperty(exports, 'createComponent', {
  enumerable: true,
  get: function () {
    return compositionApi.createComponent;
  }
});
Object.defineProperty(exports, 'createElement', {
  enumerable: true,
  get: function () {
    return compositionApi.createElement;
  }
});
Object.defineProperty(exports, 'getCurrentInstance', {
  enumerable: true,
  get: function () {
    return compositionApi.getCurrentInstance;
  }
});
Object.defineProperty(exports, 'inject', {
  enumerable: true,
  get: function () {
    return compositionApi.inject;
  }
});
Object.defineProperty(exports, 'isRef', {
  enumerable: true,
  get: function () {
    return compositionApi.isRef;
  }
});
Object.defineProperty(exports, 'onActivated', {
  enumerable: true,
  get: function () {
    return compositionApi.onActivated;
  }
});
Object.defineProperty(exports, 'onBeforeMount', {
  enumerable: true,
  get: function () {
    return compositionApi.onBeforeMount;
  }
});
Object.defineProperty(exports, 'onBeforeUnmount', {
  enumerable: true,
  get: function () {
    return compositionApi.onBeforeUnmount;
  }
});
Object.defineProperty(exports, 'onBeforeUpdate', {
  enumerable: true,
  get: function () {
    return compositionApi.onBeforeUpdate;
  }
});
Object.defineProperty(exports, 'onDeactivated', {
  enumerable: true,
  get: function () {
    return compositionApi.onDeactivated;
  }
});
Object.defineProperty(exports, 'onErrorCaptured', {
  enumerable: true,
  get: function () {
    return compositionApi.onErrorCaptured;
  }
});
Object.defineProperty(exports, 'onMounted', {
  enumerable: true,
  get: function () {
    return compositionApi.onMounted;
  }
});
Object.defineProperty(exports, 'onServerPrefetch', {
  enumerable: true,
  get: function () {
    return compositionApi.onServerPrefetch;
  }
});
Object.defineProperty(exports, 'onUnmounted', {
  enumerable: true,
  get: function () {
    return compositionApi.onUnmounted;
  }
});
Object.defineProperty(exports, 'onUpdated', {
  enumerable: true,
  get: function () {
    return compositionApi.onUpdated;
  }
});
Object.defineProperty(exports, 'provide', {
  enumerable: true,
  get: function () {
    return compositionApi.provide;
  }
});
Object.defineProperty(exports, 'reactive', {
  enumerable: true,
  get: function () {
    return compositionApi.reactive;
  }
});
Object.defineProperty(exports, 'ref', {
  enumerable: true,
  get: function () {
    return compositionApi.ref;
  }
});
Object.defineProperty(exports, 'set', {
  enumerable: true,
  get: function () {
    return compositionApi.set;
  }
});
Object.defineProperty(exports, 'toRefs', {
  enumerable: true,
  get: function () {
    return compositionApi.toRefs;
  }
});
Object.defineProperty(exports, 'watch', {
  enumerable: true,
  get: function () {
    return compositionApi.watch;
  }
});
Object.defineProperty(exports, 'watchEffect', {
  enumerable: true,
  get: function () {
    return compositionApi.watchEffect;
  }
});
exports.default = compositionApiModule;
exports.defineComponent = defineComponent;
exports.meta = meta;
exports.setSSRContext = setSSRContext;
exports.shallowSsrRef = shallowSsrRef;
exports.ssrRef = ssrRef;
exports.useAsync = useAsync;
exports.useContext = useContext;
exports.useFetch = useFetch;
exports.useMeta = useMeta;
exports.withContext = withContext;
